/* $AstecId$ */
/* ALL RIGHTS RESERVED
 *
 * This file is provided for demonstration and educational uses only.
 * Permission to use, copy, modify and distribute this file for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of KL Group not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL KL GROUP OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * RELATING TO THE USE OF THIS SOFTWARE.
 *
 *
 * XRT/field widget UIM/X integration file
 *
 *  SCCSID -- @(#)XrtFld.cl.c	1.12    98/11/05 KL Group Inc.
 */

/*****************************************************************************
				XrtFld.cl.c
*****************************************************************************/

#include <Xm/Xm.h>				/* Motif header file */
#include <Xm/XrtField.h>		/* Widget-class public header file */
#include <Xm/XrtFldManager.h>

#include <UxLib.h>

#ifdef	XtName
#undef	XtName
#endif

#include "veos.h"				/* Visual Edge object system */
#include "valuesOf.h"			/* ValuesOf & Validator functions */
#include "resource.h"
#include "resvalue.h"
#include "plist.cl.h"
#include "observer.h"

#include "XrtFld.cl.h"			/* private swidget-class header file */
#include "XrtFld_xtype.h"

/* Global variables for the XrtField class and its resource descriptors */

Class_t	UxC_XrtFldManager  	 = NULL_CLASS;  /* base class of XrtComboBox,
											   XrtSpinBox and XrtCalendar */
Class_t	UxC_XrtField		 = NULL_CLASS;
Class_t	UxC_XrtStringField	 = NULL_CLASS;
Class_t	UxC_XrtDateField	 = NULL_CLASS;
Class_t	UxC_XrtFloatField	 = NULL_CLASS;
Class_t	UxC_XrtCurrencyField	 = NULL_CLASS;
Class_t	UxC_XrtIntField 	 = NULL_CLASS;
Class_t	UxC_XrtComboBox 	 = NULL_CLASS;
Class_t	UxC_XrtSpinBox 	 	 = NULL_CLASS;
Class_t	UxC_XrtCalendar 	 = NULL_CLASS;

static void UxXrtComboBoxBuild();
static void UxXrtSpinBoxBuild();

static void
XrtFld_define_resource(n, table, class)
int 				n;
XrtFldResourceDesc	*table;
Class_t				class;
{
	binptr				prpty;
	Resource_t			*pResource;
	XrtFldResourceDesc	*pResDesc;
	Division_t			division;
	Pass_t				pass;

	pResDesc = &(table[n]);
	prpty = UxFixed_class_prop(pResDesc->rd_name,
							   class,
							   T_PNTR,
							   pResDesc->offset);
	
	division = (Division_t) pResDesc->division;
	pass = (Pass_t) pResDesc->pass;
	
	switch(pResDesc->type->datatype.type_field) {
	case XRTFLD_DOUBLE:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_UTYPE,			pResDesc->type->utype,
							 RD_XTYPE,			pResDesc->type->xtype,
							 RD_VALUESOF,		pResDesc->type->values_of,
							 RD_VALIDATOR,		pResDesc->type->validate,
							 RD_GETFUNCTION,	pResDesc->type->get_func,
							 RD_PUTFUNCTION,	pResDesc->type->put_func,
							 RD_READONLY,		pResDesc->readonly,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	case XRTFLD_COLOR:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_UTYPE,			pResDesc->type->utype,
							 RD_XTYPE,			pResDesc->type->xtype,
							 RD_VALUESOF,		pResDesc->type->values_of,
							 RD_VALIDATOR,		pResDesc->type->validate,
							 RD_EDITFUNCTION,	pResDesc->type->edit_func,
							 RD_READONLY,		pResDesc->readonly,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	case XRTFLD_CALLBACK:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	default:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_UTYPE,			pResDesc->type->utype,
							 RD_XTYPE,			pResDesc->type->xtype,
							 RD_VALUESOF,		pResDesc->type->values_of,
							 RD_VALIDATOR,		pResDesc->type->validate,
							 RD_READONLY,		pResDesc->readonly,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	}

	UxPutClassResource(class, prpty, pResource);
	pResDesc->prpty = prpty;
}

/* Function that defines the XrtFldManager class */
void
UxRegister_XrtFldManager()
{
	int		n;
	
	if (UxUndefined_class(UxC_manager)) {
		UxRegister_manager();
	}

	UxC_XrtFldManager = UxRegister_class(XRTFLDMANAGER_CLASS_NAME,
									UxC_manager,
									sizeof(UxXrtFldManager),
									sizeof(UxXrtFldManagerClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtFldManager, "UxXrtFldManager.h");
	UxPutToolKitClass(UxC_XrtFldManager, (char *) xmXrtFldManagerWidgetClass);

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtFldManager );

}


/* Function that defines the XrtField class */
void
UxRegister_XrtField()
{
	int		n;
	
	if (UxUndefined_class(UxC_text)) {
		UxRegister_text();
	}

	UxC_XrtField = UxRegister_class(XRTFLD_CLASS_NAME,
									UxC_text,
									sizeof(UxXrtField),
									sizeof(UxXrtFieldClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtField, "UxXrtField.h");
	UxPutToolKitClass(UxC_XrtField, (char *) xmXrtFieldWidgetClass);

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtField );
	
	for (n = 0; n < numfld_res; n++) {
		if (xrtfld_res_table[n].used) {
			XrtFld_define_resource(n, xrtfld_res_table, UxC_XrtField);
		}
	}

}

static KidLostOrAdded (this, child, newparent)
	swidget	this;
	swidget child;
	swidget newparent;
{
	extern int UxRecreateXWidget();

	if (UxGetWidget(this))
	{
		UxScheduleAfterEvent(UxRecreateXWidget,this);
	}
}

void
UxRegister_XrtStringField()
{
	int		n;
	
	if (UxUndefined_class(UxC_XrtField)) {
		UxRegister_XrtField();
	}

	UxC_XrtStringField = UxRegister_class(XRTSTRINGFLD_CLASS_NAME,
										  UxC_XrtField,
										  sizeof(UxXrtStringField),
										  sizeof(UxXrtStringFieldClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtStringField, "UxXrtStringField.h");
	UxPutToolKitClass(UxC_XrtStringField,
					  (char *) xmXrtStringFieldWidgetClass);
    UxPutIconBitmap(UxC_XrtStringField, "xrtStringField.xpm");

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtStringField );
	
	for (n = 0; n < numfldstr_res; n++) {
		if (xrtfldstr_res_table[n].used) {
			XrtFld_define_resource(n, xrtfldstr_res_table,
								   UxC_XrtStringField);
		}
	}

}

/* Function that defines the XrtDateField class */

void
UxRegister_XrtDateField()
{
	int		n;
	
	if (UxUndefined_class(UxC_XrtField)) {
		UxRegister_XrtField();
	}

	UxC_XrtDateField = UxRegister_class(XRTDATEFLD_CLASS_NAME,
										UxC_XrtField,
										sizeof(UxXrtDateField),
										sizeof(UxXrtDateFieldClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtDateField, "UxXrtDateField.h");
	UxPutToolKitClass(UxC_XrtDateField,
					  (char *) xmXrtDateFieldWidgetClass);
    UxPutIconBitmap(UxC_XrtDateField, "xrtDateField.xpm");

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtDateField );
	
	for (n = 0; n < numflddat_res; n++) {
		if (xrtflddat_res_table[n].used) {
			XrtFld_define_resource(n, xrtflddat_res_table,
								   UxC_XrtDateField);
		}
	}

}

/* Function that defines the XrtFloatField class */

void
UxRegister_XrtFloatField()
{
	int		n;
	
	if (UxUndefined_class(UxC_XrtField)) {
		UxRegister_XrtField();
	}

	UxC_XrtFloatField = UxRegister_class(XRTFLOATFLD_CLASS_NAME,
										 UxC_XrtField,
										 sizeof(UxXrtFloatField),
										 sizeof(UxXrtFloatFieldClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtFloatField, "UxXrtFloatField.h");
	UxPutToolKitClass(UxC_XrtFloatField,
					  (char *) xmXrtFloatFieldWidgetClass);
    UxPutIconBitmap(UxC_XrtFloatField, "xrtFloatField.xpm");

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtFloatField );
	
	for (n = 0; n < numfldflt_res; n++) {
		if (xrtfldflt_res_table[n].used) {
			XrtFld_define_resource(n, xrtfldflt_res_table,
								   UxC_XrtFloatField);
		}
	}

}

/* Function that defines the XrtCurrencyField class */

void
UxRegister_XrtCurrencyField()
{
	int		n;
	
	if (UxUndefined_class(UxC_XrtFloatField)) {
		UxRegister_XrtFloatField();
	}

	UxC_XrtCurrencyField = UxRegister_class(XRTCURRENCYFLD_CLASS_NAME,
											UxC_XrtFloatField,
											sizeof(UxXrtCurrencyField),
											sizeof(UxXrtCurrencyFieldClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtCurrencyField, "UxXrtCurrencyField.h");
	UxPutToolKitClass(UxC_XrtCurrencyField,
					  (char *) xmXrtCurrencyFieldWidgetClass);
    UxPutIconBitmap(UxC_XrtCurrencyField, "xrtCurrencyField.xpm");

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtCurrencyField );
	
}

/* Function that defines the XrtIntField class */

void
UxRegister_XrtIntField()
{
	int		n;
	
	if (UxUndefined_class(UxC_XrtField)) {
		UxRegister_XrtField();
	}

	UxC_XrtIntField = UxRegister_class(XRTINTFLD_CLASS_NAME,
									   UxC_XrtField,
									   sizeof(UxXrtIntField),
									   sizeof(UxXrtIntFieldClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtIntField, "UxXrtIntField.h");
	UxPutToolKitClass(UxC_XrtIntField,
					  (char *) xmXrtIntFieldWidgetClass);
    UxPutIconBitmap(UxC_XrtIntField, "xrtIntField.xpm");

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtIntField );
	
	for (n = 0; n < numfldint_res; n++) {
		if (xrtfldint_res_table[n].used) {
			XrtFld_define_resource(n, xrtfldint_res_table,
								   UxC_XrtIntField);
		}
	}

}

void
UxRegister_XrtComboBox()
{
	int		n;
	
	if (UxUndefined_class(UxC_XrtFldManager)) {
		UxRegister_XrtFldManager();
	}

	UxC_XrtComboBox = UxRegister_class(XRTCOMBOBOX_CLASS_NAME,
									   UxC_XrtFldManager,
									   sizeof(UxXrtComboBox),
									   sizeof(UxXrtComboBoxClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtComboBox, "UxXrtComboBox.h");
	UxPutToolKitClass(UxC_XrtComboBox, (char *) xmXrtComboBoxWidgetClass);
    UxPutIconBitmap(UxC_XrtComboBox, "xrtComboBox.xpm");

	/* METHOD REGISTRATION */
	
	UxInit_method(UxC_XrtComboBox, UxM_UxBuild, UxXrtComboBoxBuild);
	UxInit_method(UxC_XrtComboBox, UxM_UxChildAdded, KidLostOrAdded);
	UxInit_method(UxC_XrtComboBox, UxM_UxChildRemoved, KidLostOrAdded);

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtComboBox );
	
	for (n = 0; n < numfldcmb_res; n++) {
		if (xrtfldcmb_res_table[n].used) {
			XrtFld_define_resource(n, xrtfldcmb_res_table, UxC_XrtComboBox);
		}
	}

}

void
UxRegister_XrtSpinBox()
{
	int		n;
	
	if (UxUndefined_class(UxC_XrtFldManager)) {
		UxRegister_XrtFldManager();
	}

	UxC_XrtSpinBox = UxRegister_class(XRTSPINBOX_CLASS_NAME,
									  UxC_XrtFldManager,
									  sizeof(UxXrtSpinBox),
									  sizeof(UxXrtSpinBoxClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtSpinBox, "UxXrtSpinBox.h");
	UxPutToolKitClass(UxC_XrtSpinBox, (char *) xmXrtSpinBoxWidgetClass);
    UxPutIconBitmap(UxC_XrtSpinBox, "xrtSpinBox.xpm");

	/* METHOD REGISTRATION */
	
	UxInit_method(UxC_XrtSpinBox, UxM_UxBuild, UxXrtSpinBoxBuild);
	UxInit_method(UxC_XrtSpinBox, UxM_UxChildAdded, KidLostOrAdded);
	UxInit_method(UxC_XrtSpinBox, UxM_UxChildRemoved, KidLostOrAdded);

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtSpinBox );
	
	for (n = 0; n < numfldspn_res; n++) {
		if (xrtfldspn_res_table[n].used) {
			XrtFld_define_resource(n, xrtfldspn_res_table, UxC_XrtSpinBox);
		}
	}

}

void
UxRegister_XrtCalendar()
{
	int		n;
	
	if (UxUndefined_class(UxC_XrtFldManager)) {
		UxRegister_XrtFldManager();
	}

	UxC_XrtCalendar = UxRegister_class(XRTCALENDAR_CLASS_NAME,
									  UxC_XrtFldManager,
									  sizeof(UxXrtCalendar),
									  sizeof(UxXrtCalendarClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtCalendar, "UxXrtCalendar.h");
	UxPutToolKitClass(UxC_XrtCalendar, (char *) xmXrtCalendarWidgetClass);
    UxPutIconBitmap(UxC_XrtCalendar, "xrtCalendar.xpm");

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtCalendar );
	
	for (n = 0; n < numfldcal_res; n++) {
		if (xrtfldcal_res_table[n].used) {
			XrtFld_define_resource(n, xrtfldcal_res_table, UxC_XrtCalendar);
		}
	}
}

static void
UxXrtComboBoxBuild(newSwidget, manage)
swidget  newSwidget;
Boolean manage;
{
	/* Call the Superclass method  */
	(void) UxVoid_get_op(UxC_XrtFldManager, UxM_UxBuild) (newSwidget, manage);
}

static void
UxXrtSpinBoxBuild(newSwidget, manage)
swidget  newSwidget;
Boolean manage;
{
	/* Call the Superclass method  */
	(void) UxVoid_get_op(UxC_XrtFldManager, UxM_UxBuild) (newSwidget, manage);
}
