/* ALL RIGHTS RESERVED
 *
 * This file is provided for demonstration and educational uses only.
 * Permission to use, copy, modify and distribute this file for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of KL Group not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL KL GROUP OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * RELATING TO THE USE OF THIS SOFTWARE.
 *
 *
 * XRT/gauge widget UIM/X integration file
 *
 *  SCCSID -- @(#)XrtGauge_rtime.c	1.1    99/11/01 KL Group Inc.
 */

/******************************************************************************
				XrtGauge_rtime.c
******************************************************************************/

#include <X11/Intrinsic.h>
#include "utype.h"

#ifdef HASXRTGAUGE

#include	"Xm/XrtGauge.h"
#include	"XrtGauge_xtype.h"
#include	"Xrt_restable.h"

static Boolean IsConverterInstalled();

#endif /* HASXRTGAUGE */

/****
 ****   The format is to call UxrDDInstall to add a new converter.
 ****
 ****   UxrDDInstall( SquareNmajorDimension, UxUT_string, UxXT_MajorDimension );
 ****   UxrDDInstall( SquareNmakeSquare, UxUT_string, UxXt_Boolean );
 ****/


static void
XrtGauge_install_converters(table, nres)
XrtGaugeResourceDesc	*table;
int					nres;
{
#ifdef HASXRTGAUGE

	int					n;
	XrtGaugeResDataType	type;
	
	xrt_debug_enter("XrtGauge_install_converters");

	for (n = 0; n < nres; n++) {
		if (table[n].used &&
			table[n].type->convert_flag == 1 &&
			IsConverterInstalled(table[n].xt_name) == False ) {
			UxDDInstall(table[n].xt_name,
						table[n].type->utype,
						table[n].type->xtype);
		}
	}

	xrt_debug_leave("XrtGauge_install_converters");
	
#endif /* HASXRTGAUGE */
}

void
XrtGauge_rtime()
{
#ifdef HASXRTGAUGE

	int		n;
	
	xrt_debug_enter("XrtGauge_rtime");

	XrtGauge_init_resource_table();

	XrtGauge_install_converters(XrtGaCircularDial_res_table, 
		numXrtGaCircularDial_res);
	XrtGauge_install_converters(XrtGaCircularGauge_res_table,
		numXrtGaCircularGauge_res);
	XrtGauge_install_converters(XrtGaCounter_res_table,
		numXrtGaCounter_res);
	XrtGauge_install_converters(XrtGaLEDIcon_res_table,
		numXrtGaLEDIcon_res);
	XrtGauge_install_converters(XrtGaLEDNumerical_res_table,
		numXrtGaLEDNumerical_res);
	XrtGauge_install_converters(XrtGaLinearDial_res_table,
		numXrtGaLinearDial_res);
	XrtGauge_install_converters(XrtGaLinearGauge_res_table,
		numXrtGaLinearGauge_res);
	XrtGauge_install_converters(XrtGaPercent_res_table,
		numXrtGaPercent_res);
	XrtGauge_install_converters(XrtGaCenter_res_table,
		numXrtGaCenter_res);
	XrtGauge_install_converters(XrtGaCircularScale_res_table,
		numXrtGaCircularScale_res);
	XrtGauge_install_converters(XrtGaFrame_res_table,
		numXrtGaFrame_res);
	XrtGauge_install_converters(XrtGaIcon_res_table,
		numXrtGaIcon_res);
	XrtGauge_install_converters(XrtGaIndicator_res_table,
		numXrtGaIndicator_res);
	XrtGauge_install_converters(XrtGaItem_res_table,
		numXrtGaItem_res);
	XrtGauge_install_converters(XrtGaLinearScale_res_table,
		numXrtGaLinearScale_res);
	XrtGauge_install_converters(XrtGaMarker_res_table,
		numXrtGaMarker_res);
	XrtGauge_install_converters(XrtGaNeedle_res_table,
		numXrtGaNeedle_res);
	XrtGauge_install_converters(XrtGaRange_res_table,
		numXrtGaRange_res);
	XrtGauge_install_converters(XrtGaState_res_table,
		numXrtGaState_res);
	XrtGauge_install_converters(XrtGaSwitch_res_table,
		numXrtGaSwitch_res);
	XrtGauge_install_converters(XrtGaText_res_table,
		numXrtGaText_res);
	XrtGauge_install_converters(XrtGaTick_res_table,
		numXrtGaTick_res);

	xrt_debug_leave("XrtGauge_rtime");
	
#endif /* HASXRTGAUGE */
}


	/*
	 * Go through the list of multiply defined resources (lookupList)
	 * to see if converter for this type was installed.
	 * - If it is on the list and
	 *   - was not installed: install it, set the install flag to TRUE
	 *   - was installed: don't install again
	 * - If it isn't on the list : install it
	 */

static Boolean
IsConverterInstalled(xt_name)
char *xt_name;
{
#ifdef HASXRTGAUGE
	
	struct lookup {
		char 	*name;
		Boolean	installed;
	};	
	static struct lookup lookupList[] = {
		/* rb ?? add gauge mltply defined res */
		/* example: {XmNxrtGaugeAlignment, FALSE},*/
		{XmNxrtGaAlignment, FALSE},
		{XmNxrtGaAllowTransform, FALSE},
		{XmNxrtGaBorder, FALSE},
		{XmNxrtGaCircularScale, FALSE},
		{XmNxrtGaCounterList, FALSE},
		{XmNxrtGaDebug, FALSE},
		{XmNxrtGaDirection, FALSE},
		{XmNxrtGaDoubleBuffer, FALSE},
		{XmNxrtGaFrame, FALSE},
		{XmNxrtGaImage, FALSE},
		{XmNxrtGaImageLayout, FALSE},
		{XmNxrtGaImageRotate, FALSE},
		{XmNxrtGaInteractionType, FALSE},
		{XmNxrtGaIsShowing, FALSE},
		{XmNxrtGaLinearScale, FALSE},
		{XmNxrtGaOffColor, FALSE},
		{XmNxrtGaOffImage, FALSE},
		{XmNxrtGaOnColor, FALSE},
		{XmNxrtGaOnImage, FALSE},
		{XmNxrtGaOrientation, FALSE},
		{XmNxrtGaRangeList, FALSE},
		{XmNxrtGaRenderList, FALSE},
		{XmNxrtGaRepaint, FALSE},
		{XmNxrtGaScale, FALSE},
		{XmNxrtGaShape, FALSE},
		{XmNxrtGaTextList, FALSE},
		{XmNxrtGaTickList, FALSE},
		{XmNxrtGaUseRenderList, FALSE},
		{NULL, FALSE}
	};
  
	int i = 0;

	/* Number of the Motif resources is overwritten.
	 * We shouldn't try to install converters for them (they are already
	 * installed)
	 * If resource name does not begin with XrtGauge then it is not Gauge
	 * resource and we assume converter was instaled
	 */

	if (strncmp(xt_name, "xrtGa", strlen("xrtGa")) ) {
	  return TRUE;
	}
	/* Resources from lookupList give "multiple defined" warning when Ux
	 * code is generated. Following code prevents converters for being
	 * installed twice  
	 */
	for( i=0; lookupList[i].name != NULL; i++) {
		if( !strcmp( lookupList[i].name, xt_name )){
			if(lookupList[i].installed == FALSE) {
			  /* this type is on the list and has not been installed yet */
				lookupList[i].installed = TRUE;
				return FALSE;
			}
			/* this type has been installed */
			return TRUE;
		}
	}

	/* got here -> XmN(...) type was not on lookupList */
	return FALSE;

#endif /* HASXRTGAUGE */
}

