/* ALL RIGHTS RESERVED
 *
 * This file is provided for demonstration and educational uses only.
 * Permission to use, copy, modify and distribute this file for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of KL Group not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL KL GROUP OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * RELATING TO THE USE OF THIS SOFTWARE.
 *
 *
 * XRT/gear widget UIM/X integration file
 *
 * $AstecId: XrtGear.cl.c,v 1.3 1999/10/13 11:41:55 nakayama Exp $
 *  SCCSID -- @(#)XrtGear.cl.c	1.8    97/02/10 KL Group Inc.
 */

/*****************************************************************************
				XrtGear.cl.c
*****************************************************************************/

#include <Xm/Xm.h>				/* Motif header file */
#include <Xm/XrtGearAll.h>		/* Widget-class public header files */
#include <UxLib.h>

#ifdef	XtName
#undef	XtName
#endif

#include "veos.h"				/* Visual Edge object system */
#include "valuesOf.h"			/* ValuesOf & Validator functions */
#include "resource.h"
#include "resvalue.h"
#include "plist.cl.h"
#include "observer.h"
#include "UxComp.h"
#include "uxproto.h"
#include "cat_macros.h"

#include "swidget.h"
#include "plist.h"


#include <XrtGear.cl.h>		/* private swidget-class header file */

#include "XrtGear_xtype.h"

/* Global variables for the XrtGear class and its resource descriptors */

Class_t	UxC_XrtManager  	 = NULL_CLASS;  /* base class used by Aligner, Toolbar */
Class_t	UxC_XrtAligner		 = NULL_CLASS;
Class_t	UxC_XrtGLabel   	 = NULL_CLASS;
Class_t	UxC_XrtPushB    	 = NULL_CLASS;
Class_t	UxC_XrtTabM      	 = NULL_CLASS;
Class_t	UxC_XrtTabB          = NULL_CLASS;
Class_t	UxC_XrtToggleB  	 = NULL_CLASS;
Class_t	UxC_XrtToolbar  	 = NULL_CLASS;
Class_t	UxC_XrtOutliner		 = NULL_CLASS;
Class_t	UxC_XrtNodeStyle	 = NULL_CLASS;
Class_t	UxC_XrtColumn	  	 = NULL_CLASS;
Class_t	UxC_XrtProgress	  	 = NULL_CLASS;
Class_t	UxC_XrtGearCombo	 = NULL_CLASS;
Class_t	UxC_XrtPanedWindow	 = NULL_CLASS;

static void presetGadgetResourcesOnXtObject();
static char* XrtOutlinerCanAddChild ();
static void createUnmanaged();
static Widget XrtNodeStyleRealize();
static void FrameOutlineXtObject();

XrtGear_define_resource(n, table, class)
int 				n;
XrtGearResourceDesc	*table;
Class_t				class;
{
	binptr				prpty;
	Resource_t			*pResource;
	XrtGearResourceDesc	*pResDesc;
	Division_t			division;
	Pass_t				pass;

	pResDesc = &(table[n]);
	prpty = UxFixed_class_prop(pResDesc->rd_name,
							   class,
							   T_PNTR,
							   pResDesc->offset);
	
	division = (Division_t) pResDesc->division;
	pass = (Pass_t) pResDesc->pass;

	switch(pResDesc->type->datatype.type_gear) {
	case XRTGEAR_CALLBACK:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	case XRTGEAR_ICON:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_UTYPE,			pResDesc->type->utype,
							 RD_XTYPE,			pResDesc->type->xtype,
							 RD_VALUESOF,		pResDesc->type->values_of,
							 RD_VALIDATOR,		pResDesc->type->validate,
							 RD_EDITFUNCTION,	pResDesc->type->edit_func,
							 RD_GETFUNCTION,	pResDesc->type->get_func,
							 RD_PUTFUNCTION,	pResDesc->type->put_func,
							 RD_READONLY,		pResDesc->readonly,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	default:
			pResource =
				UxDefineResource(RD_NAME,			pResDesc->name,
								 RD_XTNAME,			pResDesc->xt_name,
								 RD_UTYPE,			pResDesc->type->utype,
								 RD_XTYPE,			pResDesc->type->xtype,
								 RD_VALUESOF,		pResDesc->type->values_of,
								 RD_VALIDATOR,		pResDesc->type->validate,
								 RD_EDITFUNCTION,	pResDesc->type->edit_func,
								 RD_READONLY,		pResDesc->readonly,
								 RD_DIVISION,		division,
								 RD_PASS,			pass,
								 RD_END);
	break;
	}

	UxPutClassResource(class, prpty, pResource);
	pResDesc->prpty = prpty;
}

/* Function that defines the XrtManager class */
void
UxRegister_XrtManager()
{
	int		n;
	
	if (UxUndefined_class(UxC_manager)) {
		UxRegister_manager();
	}

	UxC_XrtManager = UxRegister_class(XRTMANAGER_CLASS_NAME,
									UxC_manager,
									sizeof(UxXrtManager),
									sizeof(UxXrtManagerClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtManager, "UxXrtManager.h");
	UxPutToolKitClass(UxC_XrtManager, (char *) xmXrtManagerWidgetClass);

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtManager );

}

/* Function that defines the XrtAligner class */
void
UxRegister_XrtAligner()
{
	int		n;

	if (UxUndefined_class(UxC_XrtManager)) {
		UxRegister_XrtManager();
	}

	UxC_XrtAligner = UxRegister_class(XRTALIGNER_CLASS_NAME,
									UxC_XrtManager,
									sizeof(UxXrtAligner),
									sizeof(UxXrtAlignerClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtAligner, "UxXrtAligner.h");
	UxPutIconBitmap(UxC_XrtAligner, "xrtAligner.xpm" );
	UxPutToolKitClass(UxC_XrtAligner, (char *) xmXrtAlignerWidgetClass);

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtAligner );

	for (n = 0; n < numAlnr_res; n++) {
		if (xrtAlnr_res_table[n].used) {
			XrtGear_define_resource(n, xrtAlnr_res_table, UxC_XrtAligner);
		}
	}

}

/* Function that defines the XrtGLabel class */
void
UxRegister_XrtGLabel()
{
	int		n;
	

	if (UxUndefined_class(UxC_label)) {
		UxRegister_label();
	}

	UxC_XrtGLabel = UxRegister_class(XRTGLABEL_CLASS_NAME,
									UxC_label,
									sizeof(UxXrtGLabel),
									sizeof(UxXrtGLabelClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtGLabel, "UxXrtGLabel.h");
	UxPutToolKitClass(UxC_XrtGLabel, (char *) xmXrtGLabelWidgetClass);
	UxPutIconBitmap(UxC_XrtGLabel, "xrtGLabel.xpm" );

	/* METHOD REGISTRATION */
	
	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtGLabel );
	
	for (n = 0; n < numGLbl_res; n++) {
		if (xrtGLbl_res_table[n].used) {
			XrtGear_define_resource(n, xrtGLbl_res_table, UxC_XrtGLabel);
		}
	}

}

/* Function that defines the XrtPushB class */
void
UxRegister_XrtPushB()
{
	int		n;
	
	if (UxUndefined_class(UxC_pushButton)) {
		UxRegister_pushButton();
	}

	UxC_XrtPushB = UxRegister_class(XRTPUSHB_CLASS_NAME,
									UxC_pushButton,
									sizeof(UxXrtPushB),
									sizeof(UxXrtPushBClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtPushB, "UxXrtPushB.h");
	UxPutToolKitClass(UxC_XrtPushB, (char *) xmXrtPushButtonWidgetClass);
	UxPutIconBitmap(UxC_XrtPushB, "xrtPushB.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtPushB );
	
	for (n = 0; n <  numPB_res; n++) {
		if (xrtPB_res_table[n].used) {
			XrtGear_define_resource(n, xrtPB_res_table, UxC_XrtPushB);
		}
	}

}

/* Function that defines the XrtTabM class */

static TabKidChange(this, child, newparent)
        swidget this;
        swidget child;
        swidget newparent;
{
        extern int UxRecreateXWidget();

	if (UxGetWidget(this))
	{
	        UxScheduleAfterEvent(UxRecreateXWidget,this);
	}
}

void
UxRegister_XrtTabM()
{
	int		n;
	
	if (UxUndefined_class(UxC_manager)) {
		UxRegister_manager();
	}

	UxC_XrtTabM = UxRegister_class(XRTTABM_CLASS_NAME,
									UxC_manager,
									sizeof(UxXrtTabM),
									sizeof(UxXrtTabMClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtTabM, "UxXrtTabM.h");
	UxPutToolKitClass(UxC_XrtTabM, (char *) xmXrtTabManagerWidgetClass);
	UxPutIconBitmap(UxC_XrtTabM, "xrtTabM.xpm" );

	/* METHOD REGISTRATION */

        UxInit_method(UxC_XrtTabM, UxM_UxChildRemoved, TabKidChange);
        UxInit_method(UxC_XrtTabM, UxM_UxChildAdded, TabKidChange);

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtTabM );
	
	for (n = 0; n < numTabM_res; n++) {
		if (xrtTabM_res_table[n].used) {
			XrtGear_define_resource(n, xrtTabM_res_table, UxC_XrtTabM);
		}
	}

}

/* Function that defines the XrtTabB class */
void
UxRegister_XrtTabB()
{
	int		n;
	
	if (UxUndefined_class(UxC_pushButton)) {
		UxRegister_pushButton();
	}

	UxC_XrtTabB = UxRegister_class(XRTTABB_CLASS_NAME,
									UxC_pushButton,
									sizeof(UxXrtTabB),
									sizeof(UxXrtTabBClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtTabB, "UxXrtTabB.h");
	UxPutToolKitClass(UxC_XrtTabB, (char *) xmXrtTabButtonWidgetClass);
	UxPutIconBitmap(UxC_XrtTabB, "xrtTabB.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtTabB );
	
	for (n = 0; n < numTabB_res; n++) {
		if (xrtTabB_res_table[n].used) {
			XrtGear_define_resource(n, xrtTabB_res_table, UxC_XrtTabB);
		}
	}

}

/* Function that defines the XrtToggleB class */
void
UxRegister_XrtToggleB()
{
	int		n;
	
	if (UxUndefined_class(UxC_toggleButton)) {
		UxRegister_toggleButton();
	}

	UxC_XrtToggleB = UxRegister_class(XRTTOGGLEB_CLASS_NAME,
									UxC_toggleButton,
									sizeof(UxXrtToggleB),
									sizeof(UxXrtToggleBClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtToggleB, "UxXrtToggleB.h");
	UxPutToolKitClass(UxC_XrtToggleB, (char *) xmXrtToggleButtonWidgetClass);
	UxPutIconBitmap(UxC_XrtToggleB, "xrtToggleB.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtToggleB );
	
	for (n = 0; n < numTogB_res; n++) {
		if (xrtTogB_res_table[n].used) {
			XrtGear_define_resource(n, xrtTogB_res_table, UxC_XrtToggleB);
		}
	}

}

/* Function that defines the XrtToolbar class */
void
UxRegister_XrtToolbar()
{
	int		n;
	
	if (UxUndefined_class(UxC_XrtManager)) {
		UxRegister_XrtManager();
	}

	UxC_XrtToolbar = UxRegister_class(XRTTOOLBAR_CLASS_NAME,
									UxC_XrtManager,
									sizeof(UxXrtToolbar),
									sizeof(UxXrtToolbarClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtToolbar, "UxXrtToolbar.h");
	UxPutToolKitClass(UxC_XrtToolbar, (char *) xmXrtToolbarWidgetClass);
	UxPutIconBitmap(UxC_XrtToolbar, "xrtToolbar.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtToolbar );
	
	for (n = 0; n < numTbar_res; n++) {
		if (xrtTbar_res_table[n].used) {
			XrtGear_define_resource(n, xrtTbar_res_table, UxC_XrtToolbar);
		}
	}
	
}

/* New classes in gear 2.0 */

void
UxRegister_XrtOutliner()
{
	int		n;
	
	if (UxUndefined_class(UxC_manager)) {
		UxRegister_manager();
	}

	UxC_XrtOutliner = UxRegister_class(XRTOUTLINER_CLASS_NAME,
								 UxC_manager,
								 sizeof(UxXrtOutliner),
								 sizeof(UxXrtOutlinerClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtOutliner, "UxXrtOutliner.h" );
	UxPutToolKitClass(UxC_XrtOutliner, (char *) xmXrtOutlinerWidgetClass );
	UxPutIconBitmap(UxC_XrtOutliner, "xrtOutliner.xpm" );

	/* METHOD REGISTRATION */

	UxInit_method(UxC_XrtOutliner, UxM_UxWidgetCannotAcceptChildren, XrtOutlinerCanAddChild);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtOutliner);

	for (n = 0; n < numOutliner_res; n++) {
		if (xrtOutliner_res_table[n].used) {
			XrtGear_define_resource(n, xrtOutliner_res_table, UxC_XrtOutliner);
		}
	}

}

/* Functions that define object classes used by XrtOutliner */
void
UxRegister_XrtNodeStyle()
{
	int			n;
	
	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtNodeStyle = UxRegister_class(XRTNODESTYLE_CLASS_NAME,
					 UxC_gadget,
					 sizeof(UxXrtNodeStyle),
					 sizeof(UxXrtNodeStyleClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtNodeStyle, "UxXrtOutliner.h" );
	UxPutToolKitClass(UxC_XrtNodeStyle, (char *) xmXrtNodeStyleObjectClass);
	UxPutIconBitmap(UxC_XrtNodeStyle, "xrtNodeStyle.xpm" );

	/* METHOD REGISTRATION */
	
	UxInit_method(UxC_XrtNodeStyle, UxM_UxBuild, createUnmanaged);
	UxInit_method(UxC_XrtNodeStyle, UxM_UxRealize, XrtNodeStyleRealize);
	UxInit_method(UxC_XrtNodeStyle, UxM_UxDrawHandles, 
						FrameOutlineXtObject);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtNodeStyle);

	presetGadgetResourcesOnXtObject(UxC_XrtNodeStyle);
	
	for (n = 0; n < numNodeStyle_res; n++) {
		if (xrtNodeStyle_res_table[n].used) {
			XrtGear_define_resource(n, xrtNodeStyle_res_table, UxC_XrtNodeStyle);
		}
	}
	
}

void
UxRegister_XrtColumn()
{
	int			n;
	

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtColumn = UxRegister_class(XRTCOLUMN_CLASS_NAME,
						 UxC_gadget,
						 sizeof(UxXrtColumn),
						 sizeof(UxXrtColumnClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtColumn, "UxXrtOutliner.h" );
	UxPutToolKitClass(UxC_XrtColumn, (char *) xmXrtColumnObjectClass);
	UxPutIconBitmap(UxC_XrtColumn, "xrtColumn.xpm" );

	/* METHOD REGISTRATION */
	
	UxInit_method(UxC_XrtColumn, UxM_UxBuild, createUnmanaged);
	UxInit_method(UxC_XrtColumn, UxM_UxDrawHandles, FrameOutlineXtObject);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtColumn);

	presetGadgetResourcesOnXtObject(UxC_XrtColumn);

	for (n = 0; n < numColumn_res; n++) {
		if (xrtColumn_res_table[n].used) {
			XrtGear_define_resource(n, xrtColumn_res_table, UxC_XrtColumn);
		}
	}
	
}

/* Function that defines the XrtGearCombo class */
void
UxRegister_XrtGearCombo()
{
	int		n;
	
	if (UxUndefined_class(UxC_XrtManager)) {
		UxRegister_XrtManager();
	}

	UxC_XrtGearCombo = UxRegister_class(XRTGEARCOMBO_CLASS_NAME,
									UxC_XrtManager,
									sizeof(UxXrtGearCombo),
									sizeof(UxXrtGearComboClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtGearCombo, "UxXrtGearCombo.h");
	UxPutToolKitClass(UxC_XrtGearCombo, (char *) xmXrtGearComboWidgetClass);
	UxPutIconBitmap(UxC_XrtGearCombo, "xrtGearCombo.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtGearCombo );
	
	for (n = 0; n < numGearCombo_res; n++) {
		if (xrtGearCombo_res_table[n].used) {
			XrtGear_define_resource(n, xrtGearCombo_res_table, UxC_XrtGearCombo);
		}
	}
	
}

/* Function that defines the XrtPanedWindow class */
void
UxRegister_XrtPanedWindow()
{
	int		n;

	if (UxUndefined_class(UxC_XrtManager)) {
		UxRegister_XrtManager();
	}

	UxC_XrtPanedWindow = UxRegister_class(XRTPANEDWINDOW_CLASS_NAME,
									UxC_XrtManager,
									sizeof(UxXrtPanedWindow),
									sizeof(UxXrtPanedWindowClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtPanedWindow, "UxXrtPanedW.h");
	UxPutToolKitClass(UxC_XrtPanedWindow, (char *) xmXrtPanedWindowWidgetClass);
	UxPutIconBitmap(UxC_XrtPanedWindow, "xrtPanedW.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtPanedWindow );
	
	for (n = 0; n < numPanedWindow_res; n++) {
		if (xrtPanedWindow_res_table[n].used) {
			XrtGear_define_resource(n, xrtPanedWindow_res_table, UxC_XrtPanedWindow);
		}
	}
}

/* Function that defines the XrtProgress class */
void
UxRegister_XrtProgress()
{
	int		n;
	
	if (UxUndefined_class(UxC_primitive)) {
		UxRegister_primitive();
	}

	UxC_XrtProgress = UxRegister_class(XRTPROGRESS_CLASS_NAME,
									UxC_primitive,
									sizeof(UxXrtProgress),
									sizeof(UxXrtProgressClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtProgress, "UxXrtProgress.h");
	UxPutToolKitClass(UxC_XrtProgress, (char *) xmXrtProgressWidgetClass);
	UxPutIconBitmap(UxC_XrtProgress, "xrtProgress.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtProgress );
	
	for (n = 0; n < numProgress_res; n++) {
		if (xrtProgress_res_table[n].used) {
			XrtGear_define_resource(n, xrtProgress_res_table, UxC_XrtProgress);
		}
	}
	
}

/*
 * To create XtObject in UIM/X we have to subclass it from XmGadget.
 * To do this properly we have to predefine (and set read-only) some
 * resources available for XmGadgets but not for XtObjects
 */

static void presetGadgetResourcesOnXtObject(class)
Class_t class;
{
  Resource_t	*res;
  
  res = UxDefineResource(RD_EXAMPLE, UxGetRD_userData(class),
                                                   RD_EDITABLE, FALSE,
                                                   RD_END);
  UxPutClassResource(class, UxP_GadgetRD_userData, res);

  res = UxDefineResource(RD_EXAMPLE, UxGetRD_unitType(class),
                                                   RD_EDITABLE, FALSE,
                                                   RD_END);
  UxPutClassResource(class, UxP_GadgetRD_unitType, res);

  res = UxDefineResource(RD_EXAMPLE, UxGetRD_traversalOn(class),
                                                   RD_EDITABLE, FALSE,
                                                   RD_END);
  UxPutClassResource(class, UxP_GadgetRD_traversalOn, res);

  res = UxDefineResource(RD_EXAMPLE, UxGetRD_shadowThickness(class
),
                                                   RD_EDITABLE, FALSE,
                                                   RD_END);
  UxPutClassResource(class, UxP_GadgetRD_shadowThickness, res);

  res = UxDefineResource(RD_EXAMPLE, UxGetRD_navigationType(class)
,
                                                   RD_EDITABLE, FALSE,
                                                   RD_END);
  UxPutClassResource(class, UxP_GadgetRD_navigationType, res);

  res = UxDefineResource(RD_EXAMPLE, UxGetRD_highlightThickness(class),
                                                   RD_EDITABLE, FALSE,
                                                   RD_END);
  UxPutClassResource(class, UxP_GadgetRD_highlightThickness, res);

  res = UxDefineResource(RD_EXAMPLE, UxGetRD_highlightOnEnter(class),
                                                   RD_EDITABLE, FALSE,
                                                   RD_END);
  UxPutClassResource(class, UxP_GadgetRD_highlightOnEnter, res);

  res = UxDefineResource(RD_EXAMPLE, UxGetRD_helpCallback(class),
                                                   RD_EDITABLE, FALSE,
                                                   RD_END);
  UxPutClassResource(class, UxP_GadgetRD_helpCallback, res);

  res = UxDefineResource(RD_EXAMPLE, UxGetRD_createManaged(class),
						 RD_EDITABLE, FALSE,
						 RD_END);
  UxPutClassResource(class, UxP_ShadowWidgetRD_createManaged, res);
}

static char* XrtOutlinerCanAddChild (parent, numChildren, childClasses, children)
swidget parent;
int numChildren;
Class_t *childClasses;
swidget *children;
{
	int	 	i;
	
	if (!UxIsSubclass(parent, UxC_XrtOutliner)) {
		return("Parent must be an XrtOutliner widget");
	}

	for (i = 0; i < numChildren; i++) {
		if (childClasses[i] != UxC_XrtNodeStyle && childClasses[i] != UxC_XrtColumn ) {
			return(CGETS_KL(BAD_XRTOUTLINER_CHILD));
		}
	}
	return(NULL);
}

static void createUnmanaged(newSwidget, manage)
swidget  newSwidget;
Boolean manage;
{
	/* Put the createManaged value will want. */
  UxPutInitialString(newSwidget, UxGetRD_createManaged(newSwidget), "\"false\"");

	/* The Superclass method MUST be called! */
  (void) UxVoid_get_op(UxC_gadget, UxM_UxBuild) (newSwidget, False);
}

/*-----------------------------------------------------------------------------
 * METHOD: UxDrawHandles
 *----------------------
 * NAME:        FrameOutlineXtObject
 * INPUT:       an outline xtobject swidget (column or nodestyle) 
 * DESCRIPTION: No handles to draw for these types.
 * LAST REV:    Aug 97 fix17407 created.
 *---------------------------------------------------------------------------*/
static  void FrameOutlineXtObject(sw)
swidget sw;
{
	/* not managed un creation so no handles to draw */
}

#include <X11/IntrinsicP.h>
#include <X11/CoreP.h>

static Widget XrtNodeStyleRealize(sw)
swidget sw;
{
	Widget w;
	CorePart *core;

	w = (Widget)UxPNTR_get_op(UxC_gadget, UxM_UxRealize)(sw);
	core = (CorePart *)UxGetWidget(sw);
	core->event_table = NULL;
	return w;
}
